function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var React = require('react');
var React__default = _interopDefault(React);

function _defineProperties(target, props) {
  for (var i = 0; i < props.length; i++) {
    var descriptor = props[i];
    descriptor.enumerable = descriptor.enumerable || false;
    descriptor.configurable = true;
    if ("value" in descriptor) descriptor.writable = true;
    Object.defineProperty(target, descriptor.key, descriptor);
  }
}

function _createClass(Constructor, protoProps, staticProps) {
  if (protoProps) _defineProperties(Constructor.prototype, protoProps);
  if (staticProps) _defineProperties(Constructor, staticProps);
  return Constructor;
}

function _extends() {
  _extends = Object.assign || function (target) {
    for (var i = 1; i < arguments.length; i++) {
      var source = arguments[i];

      for (var key in source) {
        if (Object.prototype.hasOwnProperty.call(source, key)) {
          target[key] = source[key];
        }
      }
    }

    return target;
  };

  return _extends.apply(this, arguments);
}

function _inheritsLoose(subClass, superClass) {
  subClass.prototype = Object.create(superClass.prototype);
  subClass.prototype.constructor = subClass;
  subClass.__proto__ = superClass;
}

function _objectWithoutPropertiesLoose(source, excluded) {
  if (source == null) return {};
  var target = {};
  var sourceKeys = Object.keys(source);
  var key, i;

  for (i = 0; i < sourceKeys.length; i++) {
    key = sourceKeys[i];
    if (excluded.indexOf(key) >= 0) continue;
    target[key] = source[key];
  }

  return target;
}

function memberCopy(value) {
  if (Array.isArray(value)) {
    return [].concat(value);
  } else if (typeof value === "object") {
    return _extends({}, value);
  } else {
    throw new Error("Can only memberCopy() arrays and objects, got '" + String(value) + "'. Probably due to invalid useForm() value.");
  }
}

function addDistinct(arr1, arr2) {
  for (var i = 0; i < arr2.length; i++) {
    if (!arr1.includes(arr2[i])) arr1.push(arr2[i]);
  }
}

var FormState = /*#__PURE__*/function () {
  function FormState(values, defaultValues, defaultState, validator, validateOnMount, validateOnChange) {
    this.formId = ++FormState.formCounter;
    this.childMap = {};
    this.dirtyMap = {};
    this.errorMap = {};
    this.listeners = {};
    this.anyListeners = {};
    this.counter = 0;
    this.values = memberCopy(values);
    this.defaultValues = memberCopy(defaultValues);
    this._state = memberCopy(defaultState);
    this.validator = validator;
    this.validateOnMount = validateOnMount;
    this.validateOnChange = validateOnChange;
  }

  var _proto = FormState.prototype;

  _proto.setValueInternal = function setValueInternal(key, value, dirty, validate, isDefault, notifyChild, notifyParent, fireAny) {
    if (isDefault === void 0) {
      isDefault = false;
    }

    if (notifyChild === void 0) {
      notifyChild = true;
    }

    if (notifyParent === void 0) {
      notifyParent = true;
    }

    if (fireAny === void 0) {
      fireAny = true;
    }

    var valueMap = isDefault ? this.defaultValues : this.values;

    if (value === undefined) {
      if (Array.isArray(valueMap)) {
        valueMap.splice(key, 1);
      } else {
        delete valueMap[key];
      }
    } else {
      valueMap[key] = value;
    }

    this.dirtyMap[key] = dirty;

    if (notifyChild) {
      var child = this.childMap[key];

      if (child) {
        child.setValues(value, validate, isDefault, true, false);
        this.dirtyMap[key] = child.dirty;
      }
    }

    this.fireListeners(key);
    if (fireAny) this.fireAnyListeners();

    if (notifyParent && this instanceof ChildFormState) {
      this.parent.setValueInternal(this.name, Object.keys(valueMap).length > 0 ? memberCopy(valueMap) : undefined, this.dirty, validate, isDefault, false, true, true);
    }

    if (validate != null ? validate : this.validateOnChange && this.validator) this.validate();
  };

  _proto.setValue = function setValue(key, value, validate, isDefault, notifyChild, notifyParent, fireAny) {
    if (isDefault === void 0) {
      isDefault = false;
    }

    if (notifyChild === void 0) {
      notifyChild = true;
    }

    if (notifyParent === void 0) {
      notifyParent = true;
    }

    if (fireAny === void 0) {
      fireAny = true;
    }

    var other = isDefault ? this.values[key] : this.defaultValues[key];

    if (typeof value === "object" && value !== null) {
      var dirty = false;

      if (!(key in this.childMap)) {
        if (value instanceof Date && other instanceof Date) {
          dirty = (value === null || value === void 0 ? void 0 : value.getTime()) !== (other === null || other === void 0 ? void 0 : other.getTime());
        } else {
          dirty = JSON.stringify(value) !== JSON.stringify(other);
        }
      }

      this.setValueInternal(key, value, dirty, validate, isDefault, notifyChild, notifyParent, fireAny);
    } else {
      var _dirty = value !== other;

      if ((isDefault ? this.defaultValues[key] : this.values[key]) === value && this.dirtyMap[key] === _dirty) {
        return;
      }

      this.setValueInternal(key, value, _dirty, validate, isDefault, notifyChild, notifyParent, fireAny);
    }
  };

  _proto.setValues = function setValues(values, validate, isDefault, notifyChild, notifyParent) {
    if (notifyChild === void 0) {
      notifyChild = true;
    }

    if (notifyParent === void 0) {
      notifyParent = true;
    }

    if (isDefault === undefined) {
      this.setValues(values, false, true, notifyChild, notifyParent);
      isDefault = false;
    }

    var keys = Object.keys(isDefault ? this.defaultValues : this.values);
    var v = values != null ? values : {};
    addDistinct(keys, Object.keys(v));

    for (var i = keys.length - 1; i >= 0; i--) {
      var key = keys[i];
      this.setValue(key, v[key], false, isDefault, notifyChild, false, false);
    }

    this.fireAnyListeners();

    if (notifyParent && this instanceof ChildFormState) {
      if (typeof values === "object" && values !== null) {
        this.parent.setValueInternal(this.name, isDefault ? memberCopy(this.defaultValues) : memberCopy(this.values), this.dirty, validate, isDefault, false, true, true);
      } else {
        this.parent.setValueInternal(this.name, values, this.dirty, validate, isDefault, false, true, true);
      }
    }

    if (validate != null ? validate : this.validateOnChange && this.validator) this.validate();
  };

  _proto.validate = function validate() {
    try {
      var _this2 = this;

      var _temp3 = function _temp3() {
        var _r2;

        _this2.setErrors((_r2 = r) != null ? _r2 : {});

        return !_this2.error;
      };

      if (!_this2.validator) return Promise.resolve(true);

      var r = _this2.validator(_this2.values);

      var _temp4 = function () {
        if (r instanceof Promise) return Promise.resolve(r).then(function (_r) {
          r = _r;
        });
      }();

      return Promise.resolve(_temp4 && _temp4.then ? _temp4.then(_temp3) : _temp3(_temp4));
    } catch (e) {
      return Promise.reject(e);
    }
  };

  _proto.validateSync = function validateSync() {
    if (!this.validator) return true;
    var r = this.validator(this.values);
    if (r instanceof Promise) throw new Error("validateSync() was called on a form with an asynchronous validator set, please use `await form.validate()` instead.");
    this.setErrors(r != null ? r : {});
    return !this.error;
  };

  _proto.setError = function setError(key, error, notifyChild, notifyParent, fireAny) {
    if (notifyChild === void 0) {
      notifyChild = true;
    }

    if (notifyParent === void 0) {
      notifyParent = true;
    }

    if (fireAny === void 0) {
      fireAny = true;
    }

    if (typeof error !== "object" && this.errorMap[key] === error) return false;
    if (!error) delete this.errorMap[key];else this.errorMap[key] = error;

    if (notifyChild && this.childMap[key]) {
      var changed = this.childMap[key].setErrors(error != null ? error : {}, true, false);
      if (!changed && error !== undefined) return false;
    }

    this.fireListeners(key);
    if (fireAny) this.fireAnyListeners();

    if (notifyParent && this instanceof ChildFormState) {
      this.parent.setError(this.name, this.error ? memberCopy(this.errorMap) : undefined, false, true);
    }

    return true;
  };

  _proto.setErrors = function setErrors(errors, notifyChild, notifyParent) {
    if (notifyChild === void 0) {
      notifyChild = true;
    }

    if (notifyParent === void 0) {
      notifyParent = true;
    }

    var keys = Object.keys(this.errorMap);

    if (typeof errors === "string") {
      if (notifyParent && this instanceof ChildFormState) {
        this.parent.setError(this.name, errors, false, true);
      }

      errors = {};
    } else {
      addDistinct(keys, Object.keys(errors));
    }

    var changed = false;

    for (var i = keys.length; i >= 0; i--) {
      var key = keys[i];

      if (this.setError(key, errors[key], notifyChild, false, false)) {
        changed = true;
      }
    }

    if (!changed) return false;
    this.fireAnyListeners();

    if (notifyParent && this instanceof ChildFormState) {
      this.parent.setError(this.name, this.error ? memberCopy(this.errorMap) : undefined, false, true);
    }

    return true;
  };

  _proto.resetAll = function resetAll(validate, notifyChild, notifyParent) {
    if (notifyChild === void 0) {
      notifyChild = true;
    }

    if (notifyParent === void 0) {
      notifyParent = true;
    }

    this.setValues(this.defaultValues, validate != null ? validate : true, false, notifyChild, notifyParent);
  };

  _proto.reset = function reset(key, validate, notifyChild, notifyParent) {
    if (notifyChild === void 0) {
      notifyChild = true;
    }

    if (notifyParent === void 0) {
      notifyParent = true;
    }

    this.setValue(key, this.defaultValues[key], validate != null ? validate : true, false, notifyChild, notifyParent);
  };

  _proto.setState = function setState(newState, notifyChild, notifyParent) {
    var _this3 = this;

    if (notifyChild === void 0) {
      notifyChild = true;
    }

    if (notifyParent === void 0) {
      notifyParent = true;
    }

    this._state = newState;
    var c = Object.keys(this.values);
    if (notifyChild) c.forEach(function (e) {
      var _this3$childMap$e;

      return (_this3$childMap$e = _this3.childMap[e]) === null || _this3$childMap$e === void 0 ? void 0 : _this3$childMap$e.setState(newState, true, false);
    });
    c.forEach(function (e) {
      return _this3.fireListeners(e);
    });
    this.fireAnyListeners();

    if (notifyParent && this instanceof ChildFormState) {
      this.parent.setState(memberCopy(this.state), false, true);
    }
  };

  _proto.handleSubmit = function handleSubmit(handler) {
    var handle = function handle(ev) {
      try {
        var _exit2 = false;

        var _this5 = this;

        ev.preventDefault();

        if (process.env.NODE_ENV === "development") {
          var buttons = Array.from(ev.target.querySelectorAll("button"));
          var noTypeButton = buttons.find(function (e) {
            return !("type" in e.attributes);
          });

          if (noTypeButton) {
            console.error("The submitted form contains a button without a type attribute. Please populate every button in your form with either type=\"button\" or type=\"submit\".", noTypeButton);
          }
        }

        return Promise.resolve(_this5.validate()).then(function (_this4$validate) {
          if (!_this4$validate) {
            _exit2 = true;
            return;
          }

          _this5.setState(_extends({}, _this5.state, {
            isSubmitting: true
          }));

          return Promise.resolve(handler(_this5, ev)).then(function () {
            _this5.setState(_extends({}, _this5.state, {
              isSubmitting: false
            }));
          });
        });
      } catch (e) {
        return Promise.reject(e);
      }
    };

    return handle.bind(this);
  };

  _proto.listen = function listen(key, listener) {
    if (!this.listeners) this.listeners = {};
    var setters = this.listeners[key];

    if (!setters) {
      setters = {};
      this.listeners[key] = setters;
    }

    var id = "" + this.counter++;
    setters[id] = listener;
    return id;
  };

  _proto.listenAny = function listenAny(listener) {
    if (!this.anyListeners) this.anyListeners = {};
    var id = "" + this.counter++;
    this.anyListeners[id] = listener;
    return id;
  };

  _proto.ignore = function ignore(key, id) {
    if (!this.listeners) return;
    var setters = this.listeners[key];

    if (!setters) {
      console.warn("Ignore was called for no reason", key, id);
      return;
    }

    delete setters[id];
  };

  _proto.ignoreAny = function ignoreAny(id) {
    if (!this.anyListeners) return;
    delete this.anyListeners[id];
  };

  _proto.fireListeners = function fireListeners(key) {
    var a = this.listeners[key];

    if (a) {
      var l = Object.keys(a);
      l.forEach(function (e) {
        return a[e]();
      });
    }
  };

  _proto.fireAnyListeners = function fireAnyListeners() {
    var _this6 = this;

    var al = Object.keys(this.anyListeners);
    al.forEach(function (e) {
      return _this6.anyListeners[e]();
    });
  };

  _createClass(FormState, [{
    key: "state",
    get: function get() {
      return this._state;
    }
  }, {
    key: "dirty",
    get: function get() {
      var _this7 = this;

      return Object.keys(this.dirtyMap).some(function (e) {
        return _this7.dirtyMap[e];
      });
    }
  }, {
    key: "error",
    get: function get() {
      var _this8 = this;

      return Object.keys(this.errorMap).some(function (e) {
        return _this8.errorMap[e];
      });
    }
  }]);

  return FormState;
}();
FormState.formCounter = 0;
var ChildFormState = /*#__PURE__*/function (_FormState) {
  _inheritsLoose(ChildFormState, _FormState);

  function ChildFormState(parent, name) {
    var _parent$values$name, _parent$defaultValues;

    var _this9;

    _this9 = _FormState.call(this, (_parent$values$name = parent.values[name]) != null ? _parent$values$name : {}, (_parent$defaultValues = parent.defaultValues[name]) != null ? _parent$defaultValues : {}, parent.state, undefined, parent.validateOnMount, parent.validateOnChange) || this;
    _this9.parent = parent;
    _this9.name = name;
    return _this9;
  }

  return ChildFormState;
}(FormState);

function useForm(defaultValues, validator, validateOnChange, validateOnMount, defaultState) {
  if (validateOnChange === void 0) {
    validateOnChange = false;
  }

  if (validateOnMount === void 0) {
    validateOnMount = false;
  }

  var c = React.useRef(null);

  if (!c.current) {
    c.current = new FormState(defaultValues, defaultValues, defaultState != null ? defaultState : {
      isSubmitting: false
    }, validator, validateOnMount, validateOnChange);
  }

  return c.current;
}
function useObjectField(parentForm, name) {
  var c = React.useRef(null);

  if (!c.current) {
    c.current = new ChildFormState(parentForm, name);
  }

  React.useEffect(function () {
    parentForm.childMap[name] = c.current;
    c.current.name = name;
    c.current.setValues(parentForm.defaultValues[name], false, true, true, false);
    c.current.setValues(parentForm.values[name], c.current.validateOnMount, false, true, true);
    return function () {
      if (parentForm.childMap[name] === c.current) {
        delete parentForm.childMap[name];
      }
    };
  }, [parentForm, name]);
  return c.current;
}
function useListener(form, name) {
  var _form$dirtyMap$name;

  var _useState = React.useState(0),
      setRender = _useState[1];

  React.useEffect(function () {
    var id = form.listen(name, function () {
      setRender(function (e) {
        return e + 1;
      });
    });
    return function () {
      return form.ignore(name, id);
    };
  }, [form, name]);
  return {
    value: form.values[name],
    defaultValue: form.defaultValues[name],
    setValue: function setValue(value) {
      return form.setValue(name, value);
    },
    dirty: (_form$dirtyMap$name = form.dirtyMap[name]) != null ? _form$dirtyMap$name : false,
    error: form.errorMap[name],
    state: form.state,
    form: form
  };
}
function useAnyListener(form) {
  var _useState2 = React.useState(0),
      setRender = _useState2[1];

  React.useEffect(function () {
    var id = form.listenAny(function () {
      setRender(function (e) {
        return e + 1;
      });
    });
    return function () {
      form.ignoreAny(id);
    };
  }, [form]);
  return form;
}
function useArrayField(parentForm, name) {
  var form = useObjectField(parentForm, name);
  var oldLength = React.useRef(-1);

  var _useState3 = React.useState(0),
      setRender = _useState3[1];

  React.useEffect(function () {
    var id = parentForm.listen(name, function () {
      var val = parentForm.values[name];

      if (Array.isArray(val) && val.length !== oldLength.current) {
        setRender(function (i) {
          return i + 1;
        });
        oldLength.current = val.length;
      }
    });
    return function () {
      return parentForm.ignore(name, id);
    };
  }, []);
  var append = React.useCallback(function (value) {
    form.setValues([].concat(form.values, [value]), true, false);
  }, []);
  var remove = React.useCallback(function (index) {
    var newValues = [].concat(form.values);
    newValues.splice(index, 1);
    form.setValues(newValues, true, false);
  }, []);
  var clear = React.useCallback(function () {
    form.setValues([], true, false);
  }, []);
  var move = React.useCallback(function (from, to) {
    if (to === from) return;
    var newArr = [].concat(form.values);
    var target = newArr[from];
    var increment = to < from ? -1 : 1;

    for (var k = from; k !== to; k += increment) {
      newArr[k] = newArr[k + increment];
    }

    newArr[to] = target;
    form.setValues(newArr, true, false);
  }, []);
  var swap = React.useCallback(function (index, newIndex) {
    if (index === newIndex) {
      return;
    }

    var values = [].concat(form.values);
    var _ref = [values[newIndex], values[index]];
    values[index] = _ref[0];
    values[newIndex] = _ref[1];
    form.setValues(values, true, false);
  }, []);
  return {
    remove: remove,
    move: move,
    swap: swap,
    clear: clear,
    append: append,
    form: form,
    values: form.values,
    setValues: form.setValues.bind(form)
  };
}
function useTruthyListener(form, name) {
  var oldTruthy = React.useRef(!!form.values[name]);

  var _useState4 = React.useState(0),
      setRender = _useState4[1];

  React.useEffect(function () {
    var id = form.listen(name, function () {
      var thruthly = !!form.values[name];

      if (thruthly !== oldTruthy.current) {
        setRender(function (i) {
          return i + 1;
        });
        oldTruthy.current = thruthly;
      }
    });
    return function () {
      return form.ignore(name, id);
    };
  }, []);
  return !form.values[name];
}

function ArrayField(props) {
  var childForm = useArrayField(props.form, props.name);
  useTruthyListener(props.form, props.name);
  if (!props.form.values[props.name]) return null;
  return /*#__PURE__*/React__default.createElement(React__default.Fragment, null, props.render(childForm));
}
function Listener(props) {
  var _props$render, _props$render2;

  var l = useListener(props.form, props.name);
  return /*#__PURE__*/React__default.createElement(React__default.Fragment, null, (_props$render = (_props$render2 = props.render) === null || _props$render2 === void 0 ? void 0 : _props$render2.call(props, l)) != null ? _props$render : String(l.value));
}
function AnyListener(props) {
  var _props$render3, _props$render4;

  var l = useAnyListener(props.form);
  return /*#__PURE__*/React__default.createElement(React__default.Fragment, null, (_props$render3 = (_props$render4 = props.render) === null || _props$render4 === void 0 ? void 0 : _props$render4.call(props, l)) != null ? _props$render3 : JSON.stringify(l.values, null, 2));
}
function ObjectField(props) {
  var childForm = useObjectField(props.form, props.name);
  useTruthyListener(props.form, props.name);
  if (!props.form.values[props.name]) return null;
  return /*#__PURE__*/React__default.createElement(React__default.Fragment, null, props.render(childForm));
}

function Field(props) {
  var form = props.form,
      _props$as = props.as,
      as = _props$as === void 0 ? "input" : _props$as,
      serializer = props.serializer,
      dateAsNumber = props.dateAsNumber,
      setNullOnUncheck = props.setNullOnUncheck,
      setUndefinedOnUncheck = props.setUndefinedOnUncheck,
      deserializer = props.deserializer,
      hideWhenNull = props.hideWhenNull,
      disabled = props.disabled,
      className = props.className,
      style = props.style,
      errorStyle = props.errorStyle,
      dirtyStyle = props.dirtyStyle,
      innerRef = props.innerRef,
      rest = _objectWithoutPropertiesLoose(props, ["form", "as", "serializer", "dateAsNumber", "setNullOnUncheck", "setUndefinedOnUncheck", "deserializer", "hideWhenNull", "disabled", "className", "style", "errorClassName", "errorStyle", "dirtyClassName", "dirtyStyle", "innerRef"]);

  var serializeProps = {
    dateAsNumber: dateAsNumber,
    setNullOnUncheck: setNullOnUncheck,
    setUndefinedOnUncheck: setUndefinedOnUncheck,
    type: props.type,
    value: props.value
  };
  var field = useListener(form, props.name);
  if (hideWhenNull && (field.value === undefined || field.value === null)) return null;
  var v = (serializer != null ? serializer : defaultSerializer)(field.value, serializeProps);
  return React__default.createElement(as, _extends({}, rest, {
    ref: innerRef,
    checked: typeof v === "boolean" ? v : undefined,
    value: typeof v === "boolean" ? undefined : v,
    disabled: field.state.isSubmitting || disabled,
    className: (field.dirty ? "field-dirty " : "") + (field.error ? "field-error " : "") + (className || ""),
    style: _extends({}, style, field.dirty ? dirtyStyle : {}, field.error ? errorStyle : {}),
    field: field,
    onChange: function onChange(ev) {
      var v = typeof ev === "object" && "target" in ev ? props.type === "checkbox" || props.type === "radio" ? ev.target.checked : ev.target.value : ev;
      if (typeof v === "string" || typeof v === "boolean") field.setValue((deserializer != null ? deserializer : defaultDeserializer)(v, field.value, serializeProps));else field.setValue(v);
    }
  }));
}
function defaultSerializer(currentValue, props) {
  switch (props.type) {
    case "datetime-local":
    case "date":
      {
        var dateValue = currentValue;

        if (dateValue === null || dateValue === undefined || dateValue === "") {
          return "";
        }

        if (typeof dateValue === "string") {
          var ni = parseInt(dateValue);
          if (!isNaN(ni)) dateValue = ni;
        }

        var date = new Date(dateValue);

        if (!isNaN(date.getTime())) {
          var _date$toISOString$spl;

          return (_date$toISOString$spl = date === null || date === void 0 ? void 0 : date.toISOString().split("T")[0]) != null ? _date$toISOString$spl : "";
        } else {
          return "";
        }
      }

    case "radio":
      {
        return currentValue === props.value;
      }

    case "checkbox":
      {
        if (props.setNullOnUncheck) {
          return currentValue !== null;
        } else if (props.setUndefinedOnUncheck) {
          return currentValue !== undefined;
        } else if (props.value !== undefined) {
          return (Array.isArray(currentValue) ? currentValue : []).includes(props.value);
        } else {
          return !!currentValue;
        }
      }

    default:
      {
        return (currentValue != null ? currentValue : "") + "";
      }
  }
}
function defaultDeserializer(inputValue, currentValue, props) {
  switch (props.type) {
    case "number":
      {
        return parseFloat(inputValue);
      }

    case "datetime-local":
    case "date":
      {
        if (inputValue) {
          var d = new Date(inputValue);
          return props.dateAsNumber ? d.getTime() : d;
        } else {
          return null;
        }
      }

    case "radio":
      {
        if (inputValue) {
          return props.value;
        }

        return currentValue;
      }

    case "checkbox":
      {
        if (props.setNullOnUncheck || props.setUndefinedOnUncheck) {
          if (inputValue && props.value === undefined && process.env.NODE_ENV === "development") {
            console.error("Checkbox using setNullOnUncheck got checked but a value to set was not found, please provide a value to the value prop.");
          }

          return inputValue ? props.value : props.setNullOnUncheck ? null : undefined;
        } else if (props.value !== undefined) {
          var arr = Array.isArray(currentValue) ? [].concat(currentValue) : [];
          if (inputValue) arr.push(props.value);else arr.splice(arr.indexOf(props.value), 1);
          return arr;
        } else {
          return inputValue;
        }
      }

    default:
      {
        return inputValue;
      }
  }
}

function FieldError(props) {
  var form = props.form,
      _props$as = props.as,
      as = _props$as === void 0 ? React__default.Fragment : _props$as,
      transformer = props.transformer,
      innerRef = props.innerRef,
      rest = _objectWithoutPropertiesLoose(props, ["form", "as", "transformer", "innerRef"]);

  var field = useListener(form, props.name);
  if (!field.error || typeof field.error === "object") return null;
  return React__default.createElement(as, _extends({}, rest, {
    ref: innerRef,
    field: field,
    children: transformer ? transformer(field.error) : String(field.error)
  }));
}

exports.AnyListener = AnyListener;
exports.ArrayField = ArrayField;
exports.ChildFormState = ChildFormState;
exports.Field = Field;
exports.FieldError = FieldError;
exports.FormState = FormState;
exports.Listener = Listener;
exports.ObjectField = ObjectField;
exports.defaultDeserializer = defaultDeserializer;
exports.defaultSerializer = defaultSerializer;
exports.useAnyListener = useAnyListener;
exports.useArrayField = useArrayField;
exports.useForm = useForm;
exports.useListener = useListener;
exports.useObjectField = useObjectField;
exports.useTruthyListener = useTruthyListener;
//# sourceMappingURL=index.js.map
